import datetime
import logging


logger = logging.getLogger(__name__)


class FacebookDataParserError(Exception):
    pass


class FacebookDataParserCriticalError(FacebookDataParserError):
    pass


class FacebookDataBaseParser(object):
    parse_prefix = 'parse_'

    def __init__(self, data, ignore_errors=False, log_errors=False):
        self.data = data
        self.ignore_errors = ignore_errors
        self.log_errors = log_errors

    def run(self):
        params = {}
        for method in self.get_parsing_methods():
            try:
                params[method.replace(self.parse_prefix, '', 1)] = getattr(
                    self, method)()
            except FacebookDataParserCriticalError as e:
                if self.log_errors:
                    logger.error(e, exc_info=True)
                raise
            except FacebookDataParserError as e:
                if self.log_errors:
                    logger.error(e, exc_info=True)
                if not self.ignore_errors:
                    raise

        return params

    def get_parsing_methods(self):
        return [
            name for name in dir(self)
            if (name.startswith(self.parse_prefix)
                and callable(getattr(self, name)))
        ]


class FacebookDataParser(FacebookDataBaseParser):
    def parse_facebook_id(self):
        try:
            return int(self.data['id'])
        except KeyError:
            raise FacebookDataParserCriticalError("No ID in data")
        except ValueError:
            raise FacebookDataParserCriticalError("ID is not number")

    def parse_first_name(self):
        return self.data.get('first_name', None)

    def parse_last_name(self):
        return self.data.get('last_name', None)

    def parse_locale(self):
        return self.data.get('locale', None)

    def parse_username(self):
        return self.data.get('username', None)

    def parse_location_name(self):
        location = self.data.get('location', None)
        if location:
            return location.get('name', None)

    def parse_location_id(self):
        location = self.data.get('location', None)
        if location:
            return location.get('id', None)

    def parse_name(self):
        return self.data.get('name', None)

    def parse_birthday(self):
        datestr = self.data.get('birthday', None)
        if not datestr:
            return None
        try:
            return datetime.datetime.strptime(datestr, "%m/%d/%Y").date()
        except ValueError as e:
            raise FacebookDataParserError("Birthday date: %s" % e)

    def parse_raw_data(self):
        return self.data.raw_data

    def parse_gender(self):
        gender_name = self.data.get('gender', None)
        if gender_name is None:
            return None
        if gender_name in ["male", "male (hidden)"]:
            return "m"
        if gender_name in ["female", "female (hidden)"]:
            return "f"
        raise FacebookDataParserError("'%s' is not valid gender."
                                      % gender_name)

    def parse_email(self):
        return self.data.get('email', None)

    def parse_age_range_min(self):
        age_range = self.data.get('age_range', {})
        return age_range.get('min', None)

    def parse_age_range_max(self):
        age_range = self.data.get('age_range', {})
        return age_range.get('max', None)

    def parse_uses_ios(self):
        return self._parse_mobile_os('iOS')

    def parse_uses_android(self):
        return self._parse_mobile_os('Android')

    def _parse_mobile_os(self, os):
        devices = self.data.get('devices', [])
        for device in devices:
            if device['os'] == os:
                return True
        return False



TIME_ZONE = 'Europe/Warsaw'
LANGUAGE_CODE = 'pl'
SITE_ID = 1
USE_I18N = True
USE_L10N = True
USE_TZ = True
MEDIA_URL = '/media/'
STATIC_URL = '/static/'

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

SECRET_KEY = '92+lsozvq7&amp;kfl5%j1o91!5cqo5f5r@d062g&amp;bzwg2&amp;v^%s9!v'

TEMPLATE_LOADERS = (
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.views.decorators.csrf._EnsureCsrfCookie',
)

ROOT_URLCONF = 'datastore.urls'

WSGI_APPLICATION = 'datastore.wsgi.application'

PROJECT_APPS = (
    'facebook_datastore',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.gis',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'facebook_auth',
    'facebook_javascript_authentication',
    'facebook_javascript_sdk',
    'south',
)
INSTALLED_APPS += PROJECT_APPS

AUTHENTICATION_BACKENDS= (
    'django.contrib.auth.backends.ModelBackend',
    'facebook_auth.backends.FacebookJavascriptBackend',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



import logging

from django.db import models


logger = logging.getLogger(__name__)


class FacebookUserProfileManager(models.Manager):
    def create_or_update(self, parameters):
        """Parameters must include 'facebook_id' key."""
        try:
            instance = self.get(facebook_id=parameters['facebook_id'])
        except FacebookUserProfile.DoesNotExist:
            return FacebookUserProfile(**parameters)
        instance.update(parameters)
        return instance


class FacebookUserProfile(models.Model):
    GENDERS = (
        ('m', 'Male'),
        ('f', 'Female'),
    )

    user = models.OneToOneField('auth.User')

    facebook_id = models.BigIntegerField(unique=True)
    access_token = models.TextField(null=True, blank=True)

    name = models.TextField(null=True, blank=True)
    first_name = models.TextField(null=True, blank=True)
    last_name = models.TextField(null=True, blank=True)
    middle_name = models.TextField(null=True, blank=True)
    username = models.TextField(null=True, blank=True)
    email = models.EmailField(null=True, blank=True, max_length=254)

    gender = models.CharField(max_length=1, choices=GENDERS, null=True,
                              blank=True)
    locale = models.CharField(max_length=5, null=True, blank=True)
    link = models.URLField(null=True, blank=True)
    birthday = models.DateField(null=True, blank=True,
                                help_text='needs user_birthday')
    location_name = models.TextField(null=True, blank=True,
                                     help_text='needs user_location')
    location_id = models.TextField(null=True, blank=True,
                                   help_text='needs user_location')
    relationship_status = models.TextField(
        null=True, blank=True, help_text='needs user_relationship'
    )
    website = models.TextField(null=True, blank=True,
                               help_text='needs user_website')
    about_me = models.TextField(blank=True, null=True,
                                help_text='needs user_about_me')
    raw_data = models.TextField(blank=True, null=True)
    age_range_min = models.IntegerField(blank=True, null=True)
    age_range_max = models.IntegerField(blank=True, null=True)
    uses_ios = models.BooleanField(default=False)
    uses_android = models.BooleanField(default=False)

    objects = FacebookUserProfileManager()

    def update(self, parameters):
        for key, value in parameters.items():
            if hasattr(self, key):
                setattr(self, key, value)
            else:
                logger.warning("FacebookUserProfile has no '%s' attribute"
                               % key)


class FacebookUserLike(models.Model):
    user = models.ForeignKey('auth.User')
    name = models.TextField()
    category = models.CharField(max_length=250)
    facebook_id = models.BigIntegerField()
    created_time = models.DateTimeField()

    class Meta:
        unique_together = ("user", "facebook_id")


class FacebookFriend(models.Model):
    user = models.ForeignKey('auth.User')
    friend_facebook_id = models.BigIntegerField()

    class Meta:
        unique_together = (('user', 'friend_facebook_id'),)



from django import test
import json
from os import path

try:
    from unittest import mock
except ImportError:
    import mock

from facebook_datastore import engines
from facebook_datastore import factories
from facebook_datastore import graph_api
from facebook_datastore import models


def get_data_from_file(file_path):
    base_path = path.dirname(__file__)
    with open(path.join(base_path, file_path)) as data_file:
        raw_data = data_file.read()
    data = json.loads(raw_data)
    return graph_api.get_data_object(raw_data, data)


class TestUserProfileEngine(test.TestCase):
    def setUp(self):
        self.data = get_data_from_file("test_data/test_data.json")
        self.expected_facebook_id = 100002364688506
        self.facebook_user = factories.FacebookUserFactory(
            user_id=self.expected_facebook_id)
        self.facebook_user.save()

    @mock.patch('facepy.graph_api.GraphAPI.get')
    def test_if_engine_creates_profile(self, graph_api_get):
        graph_api_get.return_value = self.data

        engine = engines.UserProfileEngine(self.facebook_user)
        engine.perform()

        profile = models.FacebookUserProfile.objects.get(
            facebook_id=self.expected_facebook_id)

        self.assertEqual(self.expected_facebook_id, profile.facebook_id)
        self.assertEqual(self.data['first_name'], profile.first_name)
        self.assertEqual(self.data['last_name'], profile.last_name)
        self.assertEqual('m', profile.gender)
        self.assertEqual(self.facebook_user.id, profile.user.id)


class TestUserLikeEngine(test.TestCase):
    def setUp(self):
        self.data = get_data_from_file("test_data/test_likes_data.json")
        self.facebook_id = 100002364688506
        self.facebook_user = factories.FacebookUserFactory(
            user_id=self.facebook_id)

    @mock.patch('facepy.graph_api.GraphAPI.get')
    def test_if_engine_creates_likes(self, graph_api_get):
        graph_api_get.return_value = [self.data]
        engine = engines.UserLikeEngine(self.facebook_user)
        engine.perform()
        likes = models.FacebookUserLike.objects.filter(
            user__id=self.facebook_user.id)
        self.assertEqual(3, likes.count())
        like = likes.get(name='DajeRade.com')
        self.assertEqual('Website', like.category)



from django.conf.urls import include
from django.conf.urls import patterns
from django.conf.urls import url

from django.contrib import admin
admin.autodiscover()


urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FacebookFriend',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('friend_facebook_id', models.BigIntegerField()),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FacebookUserLike',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.TextField()),
                ('category', models.CharField(max_length=250)),
                ('facebook_id', models.BigIntegerField()),
                ('created_time', models.DateTimeField()),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FacebookUserProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('facebook_id', models.BigIntegerField(unique=True)),
                ('access_token', models.TextField(null=True, blank=True)),
                ('name', models.TextField(null=True, blank=True)),
                ('first_name', models.TextField(null=True, blank=True)),
                ('last_name', models.TextField(null=True, blank=True)),
                ('middle_name', models.TextField(null=True, blank=True)),
                ('username', models.TextField(null=True, blank=True)),
                ('email', models.EmailField(max_length=254, null=True, blank=True)),
                ('gender', models.CharField(blank=True, max_length=1, null=True, choices=[(b'm', b'Male'), (b'f', b'Female')])),
                ('locale', models.CharField(max_length=5, null=True, blank=True)),
                ('link', models.URLField(null=True, blank=True)),
                ('birthday', models.DateField(help_text=b'needs user_birthday', null=True, blank=True)),
                ('location_name', models.TextField(help_text=b'needs user_location', null=True, blank=True)),
                ('location_id', models.TextField(help_text=b'needs user_location', null=True, blank=True)),
                ('relationship_status', models.TextField(help_text=b'needs user_relationship', null=True, blank=True)),
                ('website', models.TextField(help_text=b'needs user_website', null=True, blank=True)),
                ('about_me', models.TextField(help_text=b'needs user_about_me', null=True, blank=True)),
                ('raw_data', models.TextField(null=True, blank=True)),
                ('user', models.ForeignKey(null=True, blank=True, to=settings.AUTH_USER_MODEL, unique=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='facebookuserlike',
            unique_together=set([('user', 'facebook_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='facebookfriend',
            unique_together=set([('user', 'friend_facebook_id')]),
        ),
    ]



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'FacebookUserProfile.uses_ios'
        db.add_column(u'facebook_datastore_facebookuserprofile', 'uses_ios',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)

        # Adding field 'FacebookUserProfile.uses_android'
        db.add_column(u'facebook_datastore_facebookuserprofile', 'uses_android',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'FacebookUserProfile.uses_ios'
        db.delete_column(u'facebook_datastore_facebookuserprofile', 'uses_ios')

        # Deleting field 'FacebookUserProfile.uses_android'
        db.delete_column(u'facebook_datastore_facebookuserprofile', 'uses_android')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'facebook_datastore.facebookfriend': {
            'Meta': {'unique_together': "(('user', 'friend_facebook_id'),)", 'object_name': 'FacebookFriend'},
            'friend_facebook_id': ('django.db.models.fields.BigIntegerField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'facebook_datastore.facebookuserlike': {
            'Meta': {'unique_together': "(('user', 'facebook_id'),)", 'object_name': 'FacebookUserLike'},
            'category': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'created_time': ('django.db.models.fields.DateTimeField', [], {}),
            'facebook_id': ('django.db.models.fields.BigIntegerField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'facebook_datastore.facebookuserprofile': {
            'Meta': {'object_name': 'FacebookUserProfile'},
            'about_me': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'access_token': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'age_range_max': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'age_range_min': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'birthday': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'facebook_id': ('django.db.models.fields.BigIntegerField', [], {'unique': 'True'}),
            'first_name': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '1', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'locale': ('django.db.models.fields.CharField', [], {'max_length': '5', 'null': 'True', 'blank': 'True'}),
            'location_id': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'location_name': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'middle_name': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'raw_data': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'relationship_status': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'username': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'uses_android': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'uses_ios': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'website': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['facebook_datastore']


from datastore.settings import *
from os import path
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

PROJECT_ROOT = path.abspath(path.dirname(__file__))

STATIC_ROOT = '/tmp/datastore-jenkins-test/static/'
STATIC_URL = '/static/'
MEDIA_ROOT = '/tmp/datastore-jenkins-test/media/'
MEDIA_URL = '/media/'
ADMIN_MEDIA_PREFIX = urljoin(STATIC_URL, 'admin/')

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'datastore-test-database',
    }
}

FACEBOOK_APP_ID = 123
FACEBOOK_APP_SECRET = '123'

INSTALLED_APPS += ('django_jenkins', )

JENKINS_TASKS = (
    'django_jenkins.tasks.with_coverage',
    'django_jenkins.tasks.run_pep8',
    'django_jenkins.tasks.run_pyflakes',
)

SITE_ID = 1



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('facebook_datastore', '0002_auto_20150212_1518'),
    ]

    operations = [
        migrations.AlterField(
            model_name='facebookuserprofile',
            name='about_me',
            field=models.TextField(blank=True, null=True, help_text='needs user_about_me'),
        ),
        migrations.AlterField(
            model_name='facebookuserprofile',
            name='birthday',
            field=models.DateField(blank=True, null=True, help_text='needs user_birthday'),
        ),
        migrations.AlterField(
            model_name='facebookuserprofile',
            name='gender',
            field=models.CharField(blank=True, max_length=1, choices=[('m', 'Male'), ('f', 'Female')], null=True),
        ),
        migrations.AlterField(
            model_name='facebookuserprofile',
            name='location_id',
            field=models.TextField(blank=True, null=True, help_text='needs user_location'),
        ),
        migrations.AlterField(
            model_name='facebookuserprofile',
            name='location_name',
            field=models.TextField(blank=True, null=True, help_text='needs user_location'),
        ),
        migrations.AlterField(
            model_name='facebookuserprofile',
            name='relationship_status',
            field=models.TextField(blank=True, null=True, help_text='needs user_relationship'),
        ),
        migrations.AlterField(
            model_name='facebookuserprofile',
            name='website',
            field=models.TextField(blank=True, null=True, help_text='needs user_website'),
        ),
    ]



from facepy import graph_api

from django.conf import settings


def get_graph_api(user):
    version = getattr(settings, 'FACEBOOK_API_VERSION', '2.1')
    return GraphAPI(user.access_token, version=version)


class GraphAPI(graph_api.GraphAPI):
    def _parse(self, data):
        parsed = super(GraphAPI, self)._parse(data)
        return get_data_object(data.decode('utf-8'), parsed)


def get_data_object(raw_data, data):
    class DataType(type(data)):
        __slots__ = ('raw_data')

    result = DataType(data)
    result.raw_data = raw_data
    return result



#!/usr/bin/env python
from setuptools import find_packages
from setuptools import setup
import sys

setup(
    name='facebook-datastore',
    version='3.1.2',
    description="Facebook Datastore",
    maintainer="Tomasz Wysocki",
    maintainer_email="tomasz@wysocki.info",
    install_requires=(
        'django>=1.4',
        'facepy>=1.0.6',
        'factory-boy',
        'isodate',
    ) + (('mock',) if sys.version_info.major < 3 else ()),
    packages=find_packages(),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('facebook_datastore', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='facebookuserprofile',
            name='age_range_max',
            field=models.IntegerField(null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='facebookuserprofile',
            name='age_range_min',
            field=models.IntegerField(null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='facebookuserprofile',
            name='uses_android',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='facebookuserprofile',
            name='uses_ios',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
import isodate
import logging

from django.contrib.auth.models import User
from django.utils import timezone
from facebook_datastore import graph_api
from facebook_datastore import models
from facebook_datastore import parser

logger = logging.getLogger(__name__)


class BaseEngine(object):
    """
    TODO - handle 500
    """
    def __init__(self, facebook_user):
        super(BaseEngine, self).__init__()
        self.facebook_user = facebook_user

    def should_run(self):
        return True

    def run(self):
        if self.should_run():
            self.perform()

    def perform(self):
        data = self.fetch()
        data = self.parse(data)
        self.save(data)

    def fetch(self):
        raise NotImplementedError

    def parse(self, data):
        raise NotImplementedError

    def save(self, data):
        raise NotImplementedError


class UserProfileEngine(BaseEngine):
    def fetch(self):
        graph = graph_api.get_graph_api(self.facebook_user)
        return graph.get('me?fields=id,name,first_name,middle_name,last_name,'
                         'email,gender,locale,link,birthday,location,'
                         'relationship_status,website,about,age_range,'
                         'devices')

    def parse(self, data):
        parser_instance = parser.FacebookDataParser(data=data)
        return parser_instance.run()

    def save(self, data):
        try:
            user = User.objects.get(id=self.facebook_user.id)
        except User.DoesNotExist:
            message = "UserProfileEngine missing user for facebook_user %d"
            logger.warning(message % self.facebook_user.id)
        else:
            profile, created = (models.FacebookUserProfile.objects
                                .get_or_create(user=user, defaults=data))
            if not created:
                profile.update(data)
                profile.save()


class UserLikeEngine(BaseEngine):
    def fetch(self):
        graph = graph_api.get_graph_api(self.facebook_user)
        likes = []

        response = graph.get('me/likes', True)
        for page in response:
            if 'data' in page and page['data']:
                likes += page['data']
        return likes

    def parse(self, data):
        for like in data:
            like['id'] = int(like['id'])
            if 'created_time' in like:
                like['created_time'] = isodate.parse_datetime(
                    like['created_time'])
            else:
                like['created_time'] = timezone.now()
            yield like

    def get_user(self):
        try:
            return User.objects.get(id=self.facebook_user.id)
        except User.DoesNotExist:
            message = "UserProfileEngine missing user"
            logger.warning(message,
                           extra={'facebook_user': self.facebook_user.id})
            raise

    def save(self, data):
        user = self.get_user()
        processed_likes = []
        for like in data:
            user_like = models.FacebookUserLike.objects
            if 'name' in like:
                defaults = {'name': like['name'],
                            'category': like.get('category', 'undefined'),
                            'created_time': like['created_time']}

                like, _ = user_like.get_or_create(user=user,
                                                  facebook_id=like['id'],
                                                  defaults=defaults)
                processed_likes.append(like.id)
            else:
                logger.info('Like without a name', extra={'like': like})

        removed_likes = models.FacebookUserLike.objects.filter(user=user)
        removed_likes = removed_likes.exclude(id__in=processed_likes)
        removed_likes.delete()


class FacebookFriendEngine(BaseEngine):
    def fetch(self):
        graph = graph_api.get_graph_api(self.facebook_user)
        friends = []

        response = graph.get('me/friends', True)
        for page in response:
            if 'data' in page and page['data']:
                friends += page['data']
        return friends

    def parse(self, data):
        for friend in data:
            yield int(friend['id'])

    def save(self, data):
        user = self.get_user()
        processed_friends = []
        for friend_id in data:
            friend, _ = (models.FacebookFriend.objects
                         .get_or_create(user=user,
                                        friend_facebook_id=friend_id))
            processed_friends.append(friend.id)

        removed_friends = models.FacebookFriend.objects.filter(user=user)
        removed_friends = removed_friends.exclude(id__in=processed_friends)
        removed_friends.delete()

    def get_user(self):
        try:
            return User.objects.get(id=self.facebook_user.id)
        except User.DoesNotExist:
            message = "UserProfileEngine missing user for facebook_user %d"
            logger.warning(message % self.facebook_user.id)
            raise


ENGINES_ENABLED = [UserProfileEngine, UserLikeEngine, FacebookFriendEngine]
